import tkinter as tk
from tkinter import filedialog
from keras.models import load_model
from keras.preprocessing import image
import cropping
from PIL import ImageTk
import numpy as np
import json


class GUI(tk.Tk):

    def __init__(self):
        super().__init__()
        with open('./assets/class_names.json', 'r') as file:
            self.class_names = json.load(file)
        self.img = None
        self.geometry("800x500")
        self.title("Hieroglyphs Translator")

        self.image_label = tk.Label(self)
        self.image_label.pack(pady=(10, 20))

        load_btn = tk.Button(self, text="Load Image", command=self.load_image)
        load_btn.pack(pady=5)

        read_btn = tk.Button(self, text="Translate", command=self.translate)
        read_btn.pack(pady=5)

        self.gardiner_label = tk.Label(self, text="Gardiners: ", font=("Helvetica", 14, "bold"))
        self.gardiner_label.pack(pady=(5, 20))

        self.translation_label = tk.Label(self, text="Translation: ", font=("Helvetica", 14, "bold"))
        self.translation_label.pack(pady=(5, 20))

        self.mainloop()

    def load_image(self):
        file_path = filedialog.askopenfilename()
        if file_path:
            self.img = image.load_img(file_path, color_mode="grayscale")
            photo = ImageTk.PhotoImage(self.img)
            self.image_label.config(image=photo)
            self.image_label.image = photo

    def translate(self):
        signs = cropping.crop(self.img)
        model = load_model("./assets/model.h5")
        gardiner = []
        for i in signs:
            x = image.img_to_array(i)
            x = np.expand_dims(x, axis=0)
            x = x / 255.0

            image_tensor = np.vstack([x])
            classes = model.predict(image_tensor)
            predicted_class = np.argmax(classes)
            gardiner.append(self.class_names[predicted_class])

        gardiner_text_content = "Gardiners: " + ", ".join(gardiner)
        self.gardiner_label.config(text=gardiner_text_content)

        translation_text_content = "Translation: " + cropping.find_translation(gardiner)
        self.translation_label.config(text=translation_text_content)